#/*++
#  Copyright (c) 2008, Intel Corporation
#  All rights reserved. 
# 
#  This UEFI SCT ("Software") is furnished under license and may only be used or copied
#  in accordance with the terms of that license UEFI Contributor Agreement.
#  No license, express or implied, by estoppel or otherwise, to any intellectual property
#  rights is granted by this document. The Software is subject to change without notice,
#  and should not be construed as a commitment by Intel Corporation to market, license,
#  sell or support any product or technology. Unless otherwise provided for in the license
#  under which this Software is provided, the Software is provided AS IS, with no warranties
#  of any kind, express or implied. Except as expressly permitted by the Software license,
#  neither Intel Corporation nor its suppliers assumes any responsibility or liability
#  for any errors or inaccuracies that may appear herein.
#--*/
if {[info exist MTFTP_INC] == 1} {
  return
}
set MTFTP_INC 1

Include Include/EntsProtocolTest.inc.tcl
Include Include/EntsConfigure.tcl
Include Include/EfiBootService.inc.tcl
Include EfiUtil/EfiInclude.tcl
Include MTFTP4/Include/Mtftp4Define.tcl
Include MTFTP4/Include/Mtftp4.proc.tcl
Include Include/GenericAssertionGuid.tcl
Include MTFTP4/Include/Mtftp4AssertionGuid.tcl

Include SNP/Include/Snp.inc.tcl
Include MNP/Include/Mnp.inc.tcl
Include ARP/Include/Arp.inc.tcl
Include IP4/Include/Ip4.inc.tcl
Include Udp4/Include/Udp4.inc.tcl

Func Mtftp4ServiceBinding->CreateChild {
  POINTER
  POINTER
}

Func Mtftp4ServiceBinding->DestroyChild {
  UINTN
  POINTER
}

Func Mtftp4->GetModeData {
  POINTER
  POINTER
  }

Func Mtftp4->Configure {
  POINTER 
  POINTER
  }

Func Mtftp4->GetInfo {
  POINTER
  POINTER
  POINTER
  UINT8
  POINTER
  POINTER
  POINTER
  POINTER
  }

Func Mtftp4->ParseOptions {
  UINT32 
  POINTER
  POINTER
  POINTER
  POINTER
  }

Func Mtftp4->ReadFile {
  POINTER
  UINTN
  UINTN
  UINTN
  POINTER
  }

Func Mtftp4->WriteFile {
  POINTER
  UINTN
  UINTN
  UINTN
  POINTER
  }

Func Mtftp4->ReadDirectory {
  POINTER
  UINTN
  UINTN
  UINTN
  POINTER
 }

Func Mtftp4->Poll {
  POINTER
 }

###################################################################
# EFI_MTFTP4_CONFIG_DATA
###################################################################
Struct EFI_MTFTP4_CONFIG_DATA {
  BOOLEAN                 UseDefaultSetting;
  EFI_IPv4_ADDRESS        StationIp;
  EFI_IPv4_ADDRESS        SubnetMask;
  UINT16                  LocalPort;
  EFI_IPv4_ADDRESS        GatewayIp;
  EFI_IPv4_ADDRESS        ServerIp;
  UINT16                  InitialServerPort;
  UINT16                  TryCount;
  UINT16                  TimeoutValue;
}


###################################################################
# EFI_MTFTP4_MODE_DATA
###################################################################
Struct EFI_MTFTP4_MODE_DATA {
  EFI_MTFTP4_CONFIG_DATA  ConfigData;
  UINT8                   SupportedOptionCount;
  POINTER                 SupportedOptions;
  UINT8                   UnsupportedOptionCount;
  POINTER                 UnsupportedOptions;
}


###################################################################
# EFI_MTFTP4_OVERRIDE_DATA
###################################################################
Struct EFI_MTFTP4_OVERRIDE_DATA {
  EFI_IPv4_ADDRESS        GatewayIp;
  EFI_IPv4_ADDRESS        ServerIp;
  UINT16                  ServerPort;
  UINT16                  TryCount;
  UINT16                  TimeoutValue;
}


###################################################################
# EFI_MTFTP4_OPTION
###################################################################
Struct EFI_MTFTP4_OPTION {
  POINTER                 OptionStr;
  POINTER                 ValueStr;
}


###################################################################
#
# EFI_MTFTP4_PACKET and Related Definitions
#
###################################################################
Struct EFI_MTFTP4_REQ_HEADER {
  UINT16                  OpCode;
  UINT8                   Filename(300);
}


Struct EFI_MTFTP4_OACK_HEADER {
  UINT16                  OpCode;
  UINT8                   Data(300);
}


Struct EFI_MTFTP4_DATA_HEADER {
  UINT16                  OpCode;
  UINT16                  Block;
  UINT8                   Data(2000);
}


Struct EFI_MTFTP4_DATA8_HEADER {
  UINT16                  OpCode;
  UINT64                  Block;
  UINT8                   Data(2000);
}


Struct EFI_MTFTP4_ACK8_HEADER {
  UINT16                  OpCode;
  UINT64                  Block(300);
}


Struct EFI_MTFTP4_ERROR_HEADER {
  UINT16                  OpCode;
  UINT16                  ErrorCode;
  UINT8                   ErrorMessage(300);
}


Struct EFI_MTFTP4_HEADER_ACK {
  UINT16                  Opcode;
  UINT16                  Block(300);
}

Union EFI_MTFTP4_PACKET {
  UINT16                  OpCode;
  EFI_MTFTP4_REQ_HEADER   Rrq;
  EFI_MTFTP4_REQ_HEADER   Wrq;
  EFI_MTFTP4_OACK_HEADER  Oack;
  EFI_MTFTP4_DATA_HEADER  Data;
  EFI_MTFTP4_HEADER_ACK   Ack;
  EFI_MTFTP4_DATA8_HEADER Data8;
  EFI_MTFTP4_ACK8_HEADER  Ack8;
  EFI_MTFTP4_ERROR_HEADER Error;
}


###################################################################
# EFI_MTFTP4_TOKEN
###################################################################
Struct EFI_MTFTP4_TOKEN {
  UINTN                       Status;
  UINTN                       Event;
  POINTER                     OverrideData;
  POINTER                     Filename;
  POINTER                     ModeStr;
  UINT32                      OptionCount;
  POINTER                     OptionList;
  UINT64                      BufferSize;
  POINTER                     Buffer;
  POINTER                     Context;
  POINTER                     CheckPacket;
  POINTER                     TimeoutCallback;
  POINTER                     PacketNeeded;
}