/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsTclCleanup.c
    
Abstract:

  Implementation of EMS TCL cleanup function.

--*/

#include <windows.h>
#include "EmsTclCleanup.h"

STATIC EmsTclResMng *EmsTclResStack;
STATIC HANDLE       EmsTclResStackMutex;

VOID
EmsTclResStackPush (
  EmsTclResMng *Mng
  )
/*++

Routine Description:

  Push a EMS TCL resource into stack

Arguments:

  Mng - The structure for the EMS TCL resource management

Returns:

  None

--*/
{
  WaitForSingleObject (EmsTclResStackMutex, INFINITE);
  Mng->Next       = EmsTclResStack;
  EmsTclResStack  = Mng;
  ReleaseMutex (EmsTclResStackMutex);
}

VOID
EmsTclResStackPop (
  EmsTclResMng **Mng
  )
/*++

Routine Description:

  Pop a EMS TCL resource from stack

Arguments:

  Mng - The structure for the EMS TCL resource management

Returns:

  None

--*/
{
  *Mng            = EmsTclResStack;
  EmsTclResStack  = EmsTclResStack->Next;
  (*Mng)->Next    = NULL;
}

STATIC
BOOLEAN
EmsTclResStackIsEmpty (
  VOID
  )
/*++

Routine Description:

  Check whether the EMS TCL resouce stack is empty

Arguments:

  None

Returns:

  TRUE or FALSE

--*/
{
  BOOLEAN Result;
  Result = (EmsTclResStack == NULL) ? TRUE : FALSE;
  return Result;
}

STATIC
INT32
EmsTclResCleanup (
  IN ClientData        clientData,
  IN Tcl_Interp        *Interp,
  IN INT32             Argc,
  IN CONST84 INT8      *Argv[]
  )
/*++

Routine Description:

  TCL command "EmsTclResCleanup" implementation routine  

Arguments:

  clientData  - Private data, if any.
  Interp      - TCL intepreter
  Argc        - Argument counter.
  Argv        - Argument value pointer array.

Returns:

  TCL_OK or TCL_ERROR

--*/
{
  EmsTclResMng  *Temp;
  Temp = NULL;
  WaitForSingleObject (EmsTclResStackMutex, INFINITE);
  while (!EmsTclResStackIsEmpty ()) {
    EmsTclResStackPop (&Temp);
    ReleaseMutex (EmsTclResStackMutex);
    (Temp->CleanupFunc) (Temp->Res);
    WaitForSingleObject (EmsTclResStackMutex, INFINITE);
  }

  ReleaseMutex (EmsTclResStackMutex);
  return TCL_OK;
}

VOID_P
ResTclInit (
  IN Tcl_Interp *Interp
  )
/*++

Routine Description:

  EMS cleanup related TCL initialization.

Arguments:

  Interp      - TCL intepreter.

Returns:

  None

--*/
{
  EmsTclResStack      = NULL;
  EmsTclResStackMutex = CreateMutex (NULL, FALSE, NULL);

  Tcl_CreateCommand (
    Interp,
    "EmsTclResCleanup",
    EmsTclResCleanup,
    (ClientData) NULL,
    (Tcl_CmdDeleteProc *) NULL
    );
}
