/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:
  
    EmsRivlType.h
    
Abstract:

    Incude header files for RIVL type functions

--*/

#ifndef _RIVL_TYPE_H_
#define _RIVL_TYPE_H_

#include "EmsRivlMain.h"

#define BOOLEAN_TYPE  0
#define INTN_TYPE     1
#define UINTN_TYPE    2
#define INT8_TYPE     3
#define UINT8_TYPE    4
#define INT16_TYPE    5
#define UINT16_TYPE   6
#define INT32_TYPE    7
#define UINT32_TYPE   8
#define INT64_TYPE    9
#define CHAR8_TYPE    10
#define CHAR16_TYPE   11
#define VOID_TYPE     12

RIVL_TYPE     *
RivlTypeFindByName (
  INT8 *Name
  )
/*++

Routine Description:

  Find RIVL variable's type accroding to name

Arguments:

  Name  - The type name

Returns:

  return the RIVL type found

--*/
;

VOID_P
RivlTypeRemoveByName (
  INT8 *Name
  )
/*++

Routine Description:

  Remove RIVL type accroding to name

Arguments:

  Name  - The type name

Returns:

  None

--*/
;

VOID_P
RivlTypeAdd (
  RIVL_TYPE *Type
  )
/*++

Routine Description:

  Add a new RIVL type

Arguments:

  Type  - The RIVL type to be added

Returns:

  None

--*/
;

VOID_P
RivlTypeDestroy (
  RIVL_TYPE *Type
  )
/*++

Routine Description:

  Destroy a RIVL type

Arguments:

  Type  - The RIVL type to be destroied

Returns:

  None

--*/
;

VOID_P
RivlTypeDestroyAll (
  VOID_P
  )
/*++

Routine Description:

  Destroy all the RIVL types

Arguments:

  None

Returns:

  None

--*/
;

INTERNAL_TYPE *
RivlInternalTypeFindByName (
  INT8 *Name
  )
/*++

Routine Description:

  Find a Rivl internal type accroding name

Arguments:

  Name  - The name of type

Returns:

  NULL if not found otherwise return the internal type found

--*/
;

#endif
