/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  BbTest.h

Abstract:

  This file defines the EFI Black-Box Test Protocol.

--*/

#ifndef _EFI_BB_TEST_H_
#define _EFI_BB_TEST_H_

//
// Includes
//

#include "Efi.h"
#include "EfiTest.h"
#include EFI_TEST_PROTOCOL_DEFINITION (StandardTestLibrary)

//
// EFI Black-Box Test Protocol Definitions
//

#define EFI_BB_TEST_GUID                \
  { 0x8C41CE3E, 0xB255, 0x4966, 0xB5, 0x3C, 0x39, 0x76, 0x5A, 0xE1, 0x11, 0xD0 }

#define EFI_BB_TEST_REVISION            0x00010000

#define EFI_IHV_BB_TEST_GUID           \
  { 0x27e36cde, 0xa7e7, 0x4a4a, 0x9b, 0x5f, 0xa3, 0x9a, 0x56, 0x75, 0xcb, 0x80 }

//
// Forward reference for pure ANSI compatibility
//

EFI_FORWARD_DECLARATION (EFI_BB_TEST_ENTRY);
EFI_FORWARD_DECLARATION (EFI_BB_TEST_PROTOCOL);

//
// EFI Black-Box Test Entry Point
//

typedef
EFI_STATUS
(EFIAPI *EFI_BB_ENTRY_POINT) (
  IN  EFI_BB_TEST_PROTOCOL              *This,
  IN  VOID                              *ClientInterface,
  IN  EFI_TEST_LEVEL                    TestLevel,
  IN  EFI_HANDLE                        SupportHandle
  );

//
// EFI Black-Box Test Entry
//

typedef struct _EFI_BB_TEST_ENTRY {
  EFI_BB_TEST_ENTRY                     *Next;
  EFI_GUID                              EntryId;
  CHAR16                                *Name;
  CHAR16                                *Description;
  EFI_TEST_LEVEL                        TestLevelSupportMap;
  EFI_GUID                              *SupportProtocols;
  EFI_TEST_ATTRIBUTE                    CaseAttribute;
  EFI_BB_ENTRY_POINT                    EntryPoint;
} EFI_BB_TEST_ENTRY;

//
// EFI Black-Box Test Protocol
//

typedef struct _EFI_BB_TEST_PROTOCOL {
  UINT64                                TestRevision;
  EFI_GUID                              CategoryGuid;
  CHAR16                                *Name;
  CHAR16                                *Description;
  EFI_BB_TEST_ENTRY                     *EntryList;
} EFI_BB_TEST_PROTOCOL;

//
// Global ID for EFI Black-Box Test Protocol
//

extern EFI_GUID gEfiBbTestGuid;
extern EFI_GUID gEfiIHVBbTestGuid;

#endif
