/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  TestOutputLibrary.h

Abstract:

  This file defines the EFI Test Output Library Protocol.

  This protocol will be invoked by the standard test library protocol, the test
  logging library protocol, or the other user-defined library protocols.
  Generally a test case should only invoke the standard test library protocol
  or the test logging library protocol to record the output message. It can not
  touch this test output library protocol directly.

--*/

#ifndef _EFI_TEST_OUTPUT_LIBRARY_H_
#define _EFI_TEST_OUTPUT_LIBRARY_H_

//
// Includes
//

#include "Efi.h"
#include "EfiTest.h"
#include EFI_PROTOCOL_DEFINITION (DevicePath)
#include EFI_PROTOCOL_DEFINITION (SimpleFileSystem)

//
// EFI Test Output Library Protocol Definitions
//

#define EFI_TEST_OUTPUT_LIBRARY_GUID        \
  { 0x8bfeab85, 0x83cf, 0x4c7b, 0x9e, 0xcd, 0xcf, 0x14, 0x28, 0x87, 0xe7, 0x12 }

#define EFI_TEST_OUTPUT_LIBRARY_REVISION    0x00010000

//
// Forward reference for pure ANSI compatibility
//

EFI_FORWARD_DECLARATION (EFI_TEST_OUTPUT_LIBRARY_PROTOCOL);

//
// EFI Test Output Library Protocol API - Open
//
typedef
EFI_STATUS
(EFIAPI *EFI_TOL_OPEN) (
  IN  EFI_TEST_OUTPUT_LIBRARY_PROTOCOL          *This,
  IN  EFI_DEVICE_PATH_PROTOCOL                  *DevicePath,
  IN  CHAR16                                    *FileName,
  IN  BOOLEAN                                   Overwrite,
  OUT EFI_FILE                                  **FileHandle
  )
/*++

Routine Description:

  Open an output file.

Arguments:

  This          - Test output library protocol instance.

  DevicePath    - Device path of the output file.

  FileName      - File name of the output file.

  Overwrite     - Control write operations. TRUE stands for overwrite the old
                  file, FALSE for append it.

  FileHandle    - Handle for the opened file.

Returns:

  EFI_SUCCESS if everything is correct.

--*/
;

//
// EFI Test Output Library Protocol API - Close
//
typedef
EFI_STATUS
(EFIAPI *EFI_TOL_CLOSE) (
  IN  EFI_TEST_OUTPUT_LIBRARY_PROTOCOL          *This,
  IN  EFI_FILE                                  *FileHandle
  )
/*++

Routine Description:

  Close an output file.

Arguments:

  This          - Test output library protocol instance.

  FileHandle    - File handle to be closed.

Returns:

  EFI_SUCCESS if everything is correct.

--*/
;

//
// EFI Test Output Library Protocol API - Write
//
typedef
EFI_STATUS
(EFIAPI *EFI_TOL_WRITE) (
  IN  EFI_TEST_OUTPUT_LIBRARY_PROTOCOL          *This,
  IN  EFI_FILE                                  *FileHandle,
  IN  CHAR16                                    *String
  )
/*++

Routine Description:

  Write a string to the output file.

Arguments:

  This          - Test output library protocol instance.

  FileHandle    - Handle for the opened file.

  String        - Null terminated Unicode string to be written.

Returns:

  EFI_SUCCESS if everything is correct.

--*/
;

//
// EFI Test Output Library Protocol
//

typedef struct _EFI_TEST_OUTPUT_LIBRARY_PROTOCOL {
  UINT64                                LibraryRevision;
  CHAR16                                *Name;
  CHAR16                                *Description;
  EFI_TOL_OPEN                          Open;
  EFI_TOL_CLOSE                         Close;
  EFI_TOL_WRITE                         Write;
} EFI_TEST_OUTPUT_LIBRARY_PROTOCOL;

//
// Global ID for EFI Test Output Library Protocol
//

extern EFI_GUID gEfiTestOutputLibraryGuid;

#endif
