/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/**
 *  This file is a test image for the Protocol Handler Boot Services Test
 */
/*++

Module Name:

    DeviceDriver12.c

Abstract:       

    for Protocol Handler Boot Services Black Box Test

--*/

#include "ProtocolDefinition.h"
#include "..\inc\TestDriver.h"

//
// TestDeviceDriver12:
// Consume InterfaceTestProtocol2 EXCLUSIVE;
// Produce InterfaceTestProtocol5 on the same handle
//

//
// data definition here
//
typedef struct {
  INTERFACE_FUNCTION_TEST_PROTOCOL_5    InterfaceFunctionTestProtocol5;
  EFI_DRIVER_BINDING_PROTOCOL           DriverBinding;
  EFI_HANDLE                            ControllerHandle;
} DEVICE_DRIVER_12_PRIVATE_DATA;

#define DEVICE_DRIVER_12_PRIVATE_DATA_FROM_DRIVER_BINDING(a) \
 _CR(a, DEVICE_DRIVER_12_PRIVATE_DATA, DriverBinding)
 
DEVICE_DRIVER_12_PRIVATE_DATA          *mPrivateData;

EFI_STATUS
InitializeDeviceDriver12 (
  IN EFI_HANDLE           ImageHandle,
  IN EFI_SYSTEM_TABLE     *SystemTable
  );
  
EFI_STATUS
DeviceDriver12BindingSupported (
  IN EFI_DRIVER_BINDING_PROTOCOL    *This,
  IN EFI_HANDLE                     Controller,
  IN EFI_DEVICE_PATH_PROTOCOL       *RemainingDevicePath
  );

EFI_STATUS
DeviceDriver12BindingStart (
  IN EFI_DRIVER_BINDING_PROTOCOL    *This,
  IN EFI_HANDLE                     Controller,
  IN EFI_DEVICE_PATH_PROTOCOL       *RemainingDevicePath
  );

EFI_STATUS
DeviceDriver12BindingStop (
  IN  EFI_DRIVER_BINDING_PROTOCOL    *This,
  IN  EFI_HANDLE                     Controller,
  IN  UINTN                          NumberOfChildren,
  IN  EFI_HANDLE                     *ChildHandleBuffer
  );

void
InitializeDriverBinding (
  EFI_DRIVER_BINDING_PROTOCOL *DriverBinding
  );

VOID
Protocol5ReportProtocolAttributes(
  IN INTERFACE_FUNCTION_TEST_PROTOCOL_5   *This,
  OUT PROTOCOL_ATTRIBUTES   *ProtocolAttributes
  );

VOID
InitializeInterfaceFunctionTestProtocol5 (
  INTERFACE_FUNCTION_TEST_PROTOCOL_5     *InterfaceFunctionTestProtocol5
  );

EFI_STATUS
DeviceDriver12Unload (
  IN EFI_HANDLE       ImageHandle
  );

//
// global variable for this test driver's image handle
//
EFI_DRIVER_ENTRY_POINT(InitializeDeviceDriver12)

EFI_STATUS
InitializeDeviceDriver12 (
  IN EFI_HANDLE           ImageHandle,
  IN EFI_SYSTEM_TABLE     *SystemTable
  )
{
  EFI_STATUS                            Status;  
  EFI_LOADED_IMAGE_PROTOCOL             *LoadedImageInfoPtr;
  
  EfiInitializeTestLib (ImageHandle, SystemTable);
  
  //
  // allocate memory for PrivateData
  //
  Status = gtBS->AllocatePool (
                        EfiBootServicesData,
                        sizeof (DEVICE_DRIVER_12_PRIVATE_DATA),
                        (VOID**)&mPrivateData
                        );
  if (EFI_ERROR(Status)) {
    return Status;
  }
  gtBS->SetMem (mPrivateData,sizeof (DEVICE_DRIVER_12_PRIVATE_DATA),0);
  
  InitializeDriverBinding (&mPrivateData->DriverBinding);
  
  Status = gtBS->InstallProtocolInterface (
            &ImageHandle,
            &gEfiDriverBindingProtocolGuid,
            EFI_NATIVE_INTERFACE,
            &mPrivateData->DriverBinding
            );
  mPrivateData->DriverBinding.ImageHandle = ImageHandle;
  mPrivateData->DriverBinding.DriverBindingHandle = ImageHandle;
  //
  // UnLoad Function Handler
  //  
  gtBS->HandleProtocol (
        ImageHandle, 
        &gEfiLoadedImageProtocolGuid, 
        (VOID*)&LoadedImageInfoPtr
        ); 
        
  LoadedImageInfoPtr->Unload = DeviceDriver12Unload;
  
  return EFI_SUCCESS;
  
}

EFI_STATUS
DeviceDriver12BindingSupported (
  IN EFI_DRIVER_BINDING_PROTOCOL    *This,
  IN EFI_HANDLE                     Controller,
  IN EFI_DEVICE_PATH_PROTOCOL       *RemainingDevicePath
  )  
{
  EFI_STATUS        Status;
  
  Status = gtBS->OpenProtocol (
                      Controller,
                      &mInterfaceFunctionTestProtocol2Guid,
                      NULL,
                      This->DriverBindingHandle,
                      Controller,
                      EFI_OPEN_PROTOCOL_TEST_PROTOCOL
                      );
  if (EFI_ERROR(Status)) {
    return EFI_UNSUPPORTED;
  }
  
  return EFI_SUCCESS;  
}

EFI_STATUS
DeviceDriver12BindingStart (
  IN EFI_DRIVER_BINDING_PROTOCOL    *This,
  IN EFI_HANDLE                     Controller,
  IN EFI_DEVICE_PATH_PROTOCOL       *RemainingDevicePath
  )
{
  EFI_STATUS                            Status;
  INTERFACE_FUNCTION_TEST_PROTOCOL_2    *IFTestProt2;
  DEVICE_DRIVER_12_PRIVATE_DATA         *PrivateData;
  
  PrivateData = DEVICE_DRIVER_12_PRIVATE_DATA_FROM_DRIVER_BINDING (This);
  
  Status = gtBS->OpenProtocol (
                     Controller,
                     &mInterfaceFunctionTestProtocol2Guid,
                     &IFTestProt2,
                     This->DriverBindingHandle,
                     Controller,
                     EFI_OPEN_PROTOCOL_EXCLUSIVE
                     );
  if (EFI_ERROR(Status)) {
    return Status;
  }
  InitializeInterfaceFunctionTestProtocol5 (&PrivateData->InterfaceFunctionTestProtocol5);
  
  Status = gtBS->OpenProtocol (
                  Controller,
                  &mInterfaceFunctionTestProtocol5Guid,
                  NULL,
                  This->DriverBindingHandle,
                  Controller,
                  EFI_OPEN_PROTOCOL_TEST_PROTOCOL
                  );
  if (EFI_ERROR(Status)) {
    Status = gtBS->InstallProtocolInterface (
                    &Controller,
                    &mInterfaceFunctionTestProtocol5Guid,
                    EFI_NATIVE_INTERFACE,
                    &PrivateData->InterfaceFunctionTestProtocol5
                    );
    PrivateData->ControllerHandle = Controller;
  }
  return Status;
}

EFI_STATUS
DeviceDriver12BindingStop (
  IN  EFI_DRIVER_BINDING_PROTOCOL    *This,
  IN  EFI_HANDLE                     Controller,
  IN  UINTN                          NumberOfChildren,
  IN  EFI_HANDLE                     *ChildHandleBuffer
  )
{
  DEVICE_DRIVER_12_PRIVATE_DATA          *PrivateData;
  
  PrivateData = DEVICE_DRIVER_12_PRIVATE_DATA_FROM_DRIVER_BINDING (This);
  
  gtBS->UninstallProtocolInterface (
            Controller,
            &mInterfaceFunctionTestProtocol5Guid,
            &PrivateData->InterfaceFunctionTestProtocol5
            );
  
  gtBS->CloseProtocol (
            Controller,
            &mInterfaceFunctionTestProtocol2Guid,
            This->DriverBindingHandle,
            Controller
            );
  
  return EFI_SUCCESS;
}

void
InitializeDriverBinding (
  EFI_DRIVER_BINDING_PROTOCOL *DriverBinding
  )
{
  DriverBinding->Supported            = DeviceDriver12BindingSupported;
  DriverBinding->Start                = DeviceDriver12BindingStart;
  DriverBinding->Stop                 = DeviceDriver12BindingStop;
  DriverBinding->Version              = 0x10;
  DriverBinding->ImageHandle          = NULL;
  DriverBinding->DriverBindingHandle  = NULL;
}

VOID
Protocol5ReportProtocolAttributes(
  IN INTERFACE_FUNCTION_TEST_PROTOCOL_5   *This,
  OUT PROTOCOL_ATTRIBUTES   *ProtocolAttributes
  )
{
  *ProtocolAttributes = This->ProtocolAttributes;
}

VOID
InitializeInterfaceFunctionTestProtocol5 (
  INTERFACE_FUNCTION_TEST_PROTOCOL_5     *InterfaceFunctionTestProtocol5
  )
{
  InterfaceFunctionTestProtocol5->ReportProtocolAttributes = Protocol5ReportProtocolAttributes;
  InterfaceFunctionTestProtocol5->ProtocolAttributes.ProtocolId = InterfaceFunctionTestProtocol5Id;
  InterfaceFunctionTestProtocol5->ProtocolAttributes.ReinstallField = 0;
}

/**
 *  The driver's Unload function
 *  @param  ImageHandle The test driver image handle
 *  @return EFI_SUCCESS Indicates the interface was Uninstalled
*/
EFI_STATUS
DeviceDriver12Unload (
  IN EFI_HANDLE       ImageHandle
  )
{
  //
  // free resources
  //
  gtBS->UninstallProtocolInterface (
            mPrivateData->ControllerHandle,
            &mInterfaceFunctionTestProtocol5Guid,
            &mPrivateData->InterfaceFunctionTestProtocol5
            );
  
  gtBS->CloseProtocol (
            mPrivateData->ControllerHandle,
            &mInterfaceFunctionTestProtocol2Guid,
            mPrivateData->DriverBinding.DriverBindingHandle,
            mPrivateData->ControllerHandle
            );
  gtBS->UninstallProtocolInterface (
              mPrivateData->DriverBinding.DriverBindingHandle,
              &gEfiDriverBindingProtocolGuid,
              &mPrivateData->DriverBinding
              );
  gtBS->FreePool (mPrivateData);
  
  return EFI_SUCCESS;
}