/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  HiiProtocol.h

Abstract:

  Hii protocol definitions, including HIIDatabase, HIIString, HIIFont, HIIImage protocol

Revision History

--*/
#ifndef _HII_PROTOCOL_DEF_H_
#define _HII_PROTOCOL_DEF_H_

#include "HiiDef.h"
//
// The definition for HIIFont protocol
//
EFI_FORWARD_DECLARATION (EFI_HII_FONT_PROTOCOL);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_STRING_TO_IMAGE) (
  IN CONST EFI_HII_FONT_PROTOCOL    *This,
  IN EFI_HII_OUT_FLAGS              Flags,
  IN CONST EFI_STRING               String,
  IN CONST EFI_FONT_DISPLAY_INFO    *StringInfo,
  IN OUT EFI_IMAGE_OUTPUT           **Blt,
  IN UINTN                          BltX,
  IN UINTN                          BltY,
  OUT EFI_HII_ROW_INFO              **RowInfoArray OPTIONAL,
  OUT UINTN                         *RowInfoArraySize OPTIONAL,
  OUT UINTN                         *ColumnInfoArray OPTIONAL
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_STRING_ID_TO_IMAGE) (
  IN CONST EFI_HII_FONT_PROTOCOL    *This,
  IN EFI_HII_OUT_FLAGS              Flags,
  IN EFI_HII_HANDLE                 PackageList,
  IN EFI_STRING_ID                  StringId,
  IN CONST CHAR8                    *Language,
  IN CONST EFI_FONT_DISPLAY_INFO    *StringInfo OPTIONAL,
  IN OUT EFI_IMAGE_OUTPUT           **Blt,
  IN UINTN                          BltX,
  IN UINTN                          BltY,
  OUT EFI_HII_ROW_INFO              **RowInfoArray OPTIONAL,
  OUT UINTN                         *RowInfoArraySize OPTIONAL,
  OUT UINTN                         *ColumnInfoArray OPTIONAL
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_GET_GLYPH) (
  IN CONST EFI_HII_FONT_PROTOCOL    *This,
  IN CHAR16                         Char,
  IN CONST EFI_FONT_DISPLAY_INFO    *StringInfo,
  OUT EFI_IMAGE_OUTPUT              **Blt,
  OUT UINTN                         *Baseline OPTIONAL
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_GET_FONT_INFO) (
  IN CONST EFI_HII_FONT_PROTOCOL    *This,
  IN OUT EFI_FONT_HANDLE            *FontHandle,
  IN CONST EFI_FONT_DISPLAY_INFO    *StringInfoIn,
  OUT EFI_FONT_DISPLAY_INFO         **StringInfoOut,
  IN CONST EFI_STRING               String OPTIONAL
);

typedef struct _EFI_HII_FONT_PROTOCOL {
  EFI_HII_STRING_TO_IMAGE           StringToImage;
  EFI_HII_STRING_ID_TO_IMAGE        StringIdToImage;
  EFI_HII_GET_GLYPH                 GetGlyph;
  EFI_HII_GET_FONT_INFO             GetFontInfo;
} EFI_HII_FONT_PROTOCOL;


//
// The definition for HIIString protocol
//
EFI_FORWARD_DECLARATION (EFI_HII_STRING_PROTOCOL);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_NEW_STRING) (
  IN CONST EFI_HII_STRING_PROTOCOL         *This,
  IN EFI_HII_HANDLE                        PackageList,
  OUT EFI_STRING_ID                        *StringId,
  IN CONST CHAR8                           *Language,
  IN CONST CHAR16                          *LanguageName,
  IN CONST EFI_STRING                      String,
  IN CONST EFI_FONT_INFO                   *StringFontInfo
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_GET_STRING) (
  IN CONST EFI_HII_STRING_PROTOCOL         *This,
  IN CONST CHAR8                           *Language,
  IN EFI_HII_HANDLE                        PackageList,
  IN EFI_STRING_ID                         StringId,
  OUT EFI_STRING                           String,
  IN OUT UINTN                             *StringSize,
  OUT EFI_FONT_INFO                        **StringFontInfo
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_SET_STRING) (
  IN CONST EFI_HII_STRING_PROTOCOL         *This,
  IN EFI_HII_HANDLE                        PackageList,
  IN EFI_STRING_ID                         StringId,
  IN CONST CHAR8                           *Language,
  IN CONST EFI_STRING                      String,
  IN CONST EFI_FONT_INFO                   *StringFontInfo
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_GET_LANGUAGES) (
  IN CONST EFI_HII_STRING_PROTOCOL         *This,
  IN EFI_HII_HANDLE                        PackageList,
  IN OUT CHAR8                             *Languages,
  IN OUT UINTN                             *LanguagesSize
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_GET_2ND_LANGUAGES) (
  IN CONST EFI_HII_STRING_PROTOCOL         *This,
  IN EFI_HII_HANDLE                        PackageList,
  IN CONST CHAR8                           *FirstLanguage,
  IN OUT CHAR8                             *SecondLanguages,
  IN OUT UINTN                             *SecondLanguagesSize
);

typedef struct _EFI_HII_STRING_PROTOCOL {
  EFI_HII_NEW_STRING                    NewString;
  EFI_HII_GET_STRING                    GetString;
  EFI_HII_SET_STRING                    SetString;
  EFI_HII_GET_LANGUAGES                 GetLanguages;
  EFI_HII_GET_2ND_LANGUAGES             GetSecondaryLanguages;
} EFI_HII_STRING_PROTOCOL;


//
// The definition for HIIImage protocol
//
EFI_FORWARD_DECLARATION (EFI_HII_IMAGE_PROTOCOL);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_NEW_IMAGE) (
  IN CONST EFI_HII_IMAGE_PROTOCOL    *This,
  IN EFI_HII_HANDLE                  PackageList,
  OUT EFI_IMAGE_ID                   *ImageId,
  IN CONST EFI_IMAGE_INPUT           *Image
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_GET_IMAGE) (
  IN CONST EFI_HII_IMAGE_PROTOCOL    *This,
  IN EFI_HII_HANDLE                  PackageList,
  IN EFI_IMAGE_ID                    ImageId,
  OUT EFI_IMAGE_INPUT                *Image
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_SET_IMAGE) (
  IN CONST EFI_HII_IMAGE_PROTOCOL    *This,
  IN EFI_HII_HANDLE                  PackageList,
  IN EFI_IMAGE_ID                    ImageId,
  IN CONST EFI_IMAGE_INPUT           *Image
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_DRAW_IMAGE) (
  IN CONST EFI_HII_IMAGE_PROTOCOL    *This,
  IN EFI_HII_DRAW_FLAGS              Flags,
  IN CONST EFI_IMAGE_INPUT           *Image,
  IN OUT EFI_IMAGE_OUTPUT            **Blt,
  IN UINTN                           BltX,
  IN UINTN                           BltY
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_DRAW_IMAGE_ID) (
  IN CONST EFI_HII_IMAGE_PROTOCOL    *This,
  IN EFI_HII_DRAW_FLAGS              Flags,
  IN EFI_HII_HANDLE                  PackageList,
  IN EFI_IMAGE_ID                    ImageId,
  IN OUT EFI_IMAGE_OUTPUT            **Blt,
  IN UINTN                           BltX,
  IN UINTN                           BltY
);

typedef struct _EFI_HII_IMAGE_PROTOCOL {
  EFI_HII_NEW_IMAGE                  NewImage;
  EFI_HII_GET_IMAGE                  GetImage;
  EFI_HII_SET_IMAGE                  SetImage;
  EFI_HII_DRAW_IMAGE                 DrawImage;
  EFI_HII_DRAW_IMAGE_ID              DrawImageId;
} EFI_HII_IMAGE_PROTOCOL;


//
// The definition for HIIDatabase protocol
//
EFI_FORWARD_DECLARATION (EFI_HII_DATABASE_PROTOCOL);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_DATABASE_NEW_PACK) (
  IN CONST EFI_HII_DATABASE_PROTOCOL      *This,
  IN CONST EFI_HII_PACKAGE_LIST_HEADER    *PackageList,
  IN CONST EFI_HANDLE                     DriverHandle,
  OUT EFI_HII_HANDLE                      *Handle
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_DATABASE_REMOVE_PACK) (
  IN CONST EFI_HII_DATABASE_PROTOCOL    *This,
  IN EFI_HII_HANDLE                     Handle
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_DATABASE_UPDATE_PACK) (
  IN CONST EFI_HII_DATABASE_PROTOCOL      *This,
  IN EFI_HII_HANDLE                       Handle,
  IN CONST EFI_HII_PACKAGE_LIST_HEADER    *PackageList
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_DATABASE_LIST_PACKS) (
  IN CONST EFI_HII_DATABASE_PROTOCOL    *This,
  IN UINT8                              PackageType,
  IN CONST EFI_GUID                     *PackageGuid,
  IN OUT UINTN                          *HandleBufferLength,
  OUT EFI_HII_HANDLE                    *Handle
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_DATABASE_EXPORT_PACKS) (
  IN CONST EFI_HII_DATABASE_PROTOCOL    *This,
  IN EFI_HII_HANDLE                     Handle,
  IN OUT UINTN                          *BufferSize,
  OUT EFI_HII_PACKAGE_LIST_HEADER       *Buffer
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_DATABASE_REGISTER_NOTIFY) (
  IN CONST EFI_HII_DATABASE_PROTOCOL    *This,
  IN UINT8                              PackageType,
  IN CONST EFI_GUID                     *PackageGuid,
  IN CONST EFI_HII_DATABASE_NOTIFY      PackageNotifyFn,
  IN EFI_HII_DATABASE_NOTIFY_TYPE       NotifyType,
  OUT EFI_HANDLE                        *NotifyHandle
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_DATABASE_UNREGISTER_NOTIFY) (
  IN CONST EFI_HII_DATABASE_PROTOCOL    *This,
  IN EFI_HANDLE                         NotificationHandle
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_FIND_KEYBOARD_LAYOUTS) (
  IN EFI_HII_DATABASE_PROTOCOL    *This,
  IN OUT UINT16                   *KeyGuidBufferLength,
  OUT EFI_GUID                    *KeyGuidBuffer
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_SET_KEYBOARD_LAYOUT) (
  IN EFI_HII_DATABASE_PROTOCOL             *This,
  IN EFI_GUID                              *KeyGuid
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_GET_KEYBOARD_LAYOUT) (
  IN EFI_HII_DATABASE_PROTOCOL    *This,
  IN EFI_GUID                     *KeyGuid,
  IN OUT UINT16                   *KeyboardLayoutLength,
  OUT EFI_HII_KEYBOARD_LAYOUT     *KeyboardLayout
);

typedef
EFI_STATUS
(EFIAPI *EFI_HII_DATABASE_GET_PACK_HANDLE) (
  IN EFI_HII_DATABASE_PROTOCOL    *This,
  IN EFI_HII_HANDLE               PackageListHandle,
  OUT EFI_HANDLE                  *DriverHandle
);

typedef struct _EFI_HII_DATABASE_PROTOCOL {
  EFI_HII_DATABASE_NEW_PACK            NewPackageList;
  EFI_HII_DATABASE_REMOVE_PACK         RemovePackageList;
  EFI_HII_DATABASE_UPDATE_PACK         UpdatePackageList;
  EFI_HII_DATABASE_LIST_PACKS          ListPackageLists;
  EFI_HII_DATABASE_EXPORT_PACKS        ExportPackageLists;
  EFI_HII_DATABASE_REGISTER_NOTIFY     RegisterPackageNotify;
  EFI_HII_DATABASE_UNREGISTER_NOTIFY   UnregisterPackageNotify;
  EFI_HII_FIND_KEYBOARD_LAYOUTS        FindKeyboardLayouts;
  EFI_HII_GET_KEYBOARD_LAYOUT          GetKeyboardLayout;
  EFI_HII_SET_KEYBOARD_LAYOUT          SetKeyboardLayout;
  EFI_HII_DATABASE_GET_PACK_HANDLE     GetPackageListHandle;
} EFI_HII_DATABASE_PROTOCOL;

#endif