/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  hashesTest.h

Abstract:
  
  This driver supports platform security service

--*/

#ifndef _HASH_STUB_ARCH_PROTOCOL_H
#define _HASH_STUB_ARCH_PROTOCOL_H

#include "Tiano.h"
#include "EfiDriverLib.h"
#include "hashes.h"


//
// Function prototypes
//

EFI_STATUS
EFIAPI
HashesStubInitialize (
  IN EFI_HANDLE        ImageHandle,
  IN EFI_SYSTEM_TABLE  *SystemTable
  );
  

EFI_FORWARD_DECLARATION (EFI_HASH_SERVICE_BINDING_PROTOCOL);


typedef
EFI_STATUS
(EFIAPI EFI_SERVICE_BINDING_CREATE_CHILD) (
	IN EFI_HASH_SERVICE_BINDING_PROTOCOL *This,
	IN OUT EFI_HANDLE *ChildHandle
);

typedef
EFI_STATUS
(EFIAPI EFI_SERVICE_BINDING_DESTROY_CHILD) (
	IN EFI_HASH_SERVICE_BINDING_PROTOCOL *This,
	IN EFI_HANDLE ChildHandle
);

typedef struct _HASH_PRIVATE_DATA {
	EFI_HASH_PROTOCOL hashServiceProtocol;
	EFI_HANDLE ChildHandle;
	EFI_HANDLE HashInterface;
	EFI_HASH_STATE hashState;
	int NumberCreated;
	int Destroy;
	EFI_HANDLE ImageHandle;
	EFI_HANDLE HashDriverBindingHandle;	
} HASH_PRIVATE_DATA;

typedef struct _EFI_HASH_SERVICE_BINDING_PROTOCOL {
	EFI_SERVICE_BINDING_CREATE_CHILD *CreateChild;
	EFI_SERVICE_BINDING_DESTROY_CHILD *DestroyChild;
	HASH_PRIVATE_DATA PrivateData;
} EFI_HASH_SERVICE_BINDING_PROTOCOL;

#define HASH_PRIVATE_DATA_FROM_SERVICE_BINDING_THIS(x) (&x->PrivateData)
#define HASH_PRIVATE_DATA_FROM_HASH_THIS(x) (HASH_PRIVATE_DATA*)(x)

#define TESTA    



#ifdef DEBUG_HASHTEST
	#define my_printf(x)  Print	
	#define my_logMsgPrintf  Print
#else
	#define my_printf(x) // Print	
	#define my_logMsgPrintf  Print
#endif


#define logMsg(_guid,_title,_message,_result,_maskRes) \
  StandardLib->RecordAssertion (StandardLib,\
                    ( (_maskRes==_result) ? EFI_TEST_ASSERTION_PASSED : EFI_TEST_ASSERTION_FAILED ),\
                   _guid,\
                   L"EFI Hash Service Protocol Testing ",\
                   L" - %a\n  %a\n  %a:%d\n  Status(0x%x) - %r",\
                   _title, _message, \
                   __FILE__, (UINTN)__LINE__,\
                   _result, ( (_maskRes==_result) ? EFI_SUCCESS : EFI_UNSUPPORTED )\
                   );

extern EFI_GUID gEfiHashesArchProtocolGuid;
extern EFI_GUID gEfiHashesBindingProtocolGuid;

EFI_STATUS
EFIAPI
HashServiceBindingCreateChildDemo (
	IN EFI_HASH_SERVICE_BINDING_PROTOCOL *_this,
	EFI_HANDLE *ChildHandle,
	EFI_HASH_PROTOCOL **Hash
);
EFI_STATUS
EFIAPI
HashServiceBindingDestoryChildDemo (
	IN EFI_HASH_SERVICE_BINDING_PROTOCOL *_this,
	EFI_HANDLE ChildHandle
);
EFI_STATUS HashProtocolTestGetSize(EFI_HASH_PROTOCOL *This);
EFI_STATUS HashProtocolTestHash(EFI_HASH_PROTOCOL *This);

#endif
