/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  MiscRuntimeServicesBBTestMain.c

Abstract:

  Test Driver of Miscellaneous Runtime Services

--*/

#include "MiscRuntimeServicesBBTestMain.h"

#if (EFI_SPECIFICATION_VERSION >= 0x00020000)
EFI_GUID mEfiCapsuleHeaderGuid = EFI_CAPSULE_GUID;
#endif

EFI_TPL TplArray [TPL_ARRAY_SIZE] = {
  EFI_TPL_APPLICATION,
  EFI_TPL_CALLBACK,
  EFI_TPL_NOTIFY
};

EFI_BB_TEST_PROTOCOL_FIELD gBBTestProtocolField = {
  MISC_RUNTIME_SERVICES_TEST_REVISION,
  MISC_RUNTIME_SERVICES_TEST_GUID,
  L"Misc Runtime Services Test",
  L"UEFI 2.0 Misc Runtime Services Black-Box Test"
};

EFI_GUID gSupportProtocolGuid1[2] = {
  EFI_STANDARD_TEST_LIBRARY_GUID,
  EFI_NULL_GUID
};

EFI_GUID gSupportProtocolGuid2[3] = {
  EFI_STANDARD_TEST_LIBRARY_GUID,
  EFI_TEST_RECOVERY_LIBRARY_GUID,
  EFI_NULL_GUID
};

EFI_BB_TEST_ENTRY_FIELD gBBTestEntryField[] = {
  {
    MISC_RUNTIME_SERVICES_RESETSYSTEM_INTERFACE_GUID,
    L"ResetSystem_Func",
    L"Interface Test for ResetSystem",
    EFI_TEST_LEVEL_DEFAULT,
    gSupportProtocolGuid2,
    EFI_TEST_CASE_AUTO,
    BBTestResetSystemInterfaceTest
  },

#if (EFI_SPECIFICATION_VERSION >= 0x00020000)
  {
    MISC_RUNTIME_SERVICES_QUERYCAPSULEBILITIES_INTERFACE_TEST_GUID,
    L"QueryCapsuleCapabilities_Func",
    L"Interface Test for QueryCapsuleCapabilities",
    EFI_TEST_LEVEL_DEFAULT,
    gSupportProtocolGuid2,
    EFI_TEST_CASE_AUTO,
    BBTestQueryCapsuleCapabilitiesTest
  },

  {
    MISC_RUNTIME_SERVICES_UPDATECAPSULE_COFORMANCE_TEST_GUID,
    L"UpdateCapsule_Conf",
    L"Conformance Test for UpdateCapsule",
    EFI_TEST_LEVEL_DEFAULT,
    gSupportProtocolGuid2,
    EFI_TEST_CASE_AUTO,
    BBTestUpdateCapsuleConformanceTest
  },

  {
    MISC_RUNTIME_SERVICES_QUERYCAPSULEBILITIES_CONFORMANCE_TEST_GUID,
    L"QueryCapsuleCapabilities_Conf",
    L"Conformance Test for QueryCapsuleCapabilities",
    EFI_TEST_LEVEL_DEFAULT,
    gSupportProtocolGuid2,
    EFI_TEST_CASE_AUTO,
    BBTestQueryCapsuleCapabilitiesConformanceTest
  },
#endif

#ifdef EFI_TEST_EXHAUSTIVE
  {
    MISC_RUNTIME_SERVICES_RESETSYSTEM_MANUAL_GUID,
    L"ResetSystem_Func_Manual",
    L"Manual Test for ResetSystem",
    EFI_TEST_LEVEL_DEFAULT,
    gSupportProtocolGuid2,
    EFI_TEST_CASE_MANUAL,
    BBTestResetSystemManualTest
  },
  {
    MISC_RUNTIME_SERVICES_RESETSYSTEM_STRESS_TEST_GUID,
    L"ResetSystem_Stress",
    L"Stress Test for ResetSystem",
    EFI_TEST_LEVEL_EXHAUSTIVE,
    gSupportProtocolGuid2,
    EFI_TEST_CASE_AUTO,
    BBTestResetSystemStressTest
  },
  {
    MISC_RUNTIME_SERVICES_RESETSYSTEM_MANUAL_STRESS_TEST_GUID,
    L"ResetSystem_Stress_Manual",
    L"Manual Stress Test for ResetSystem",
    EFI_TEST_LEVEL_EXHAUSTIVE,
    gSupportProtocolGuid2,
    EFI_TEST_CASE_MANUAL,
    BBTestResetSystemManualStressTest
  },
#endif

  EFI_NULL_GUID
};

EFI_BB_TEST_PROTOCOL *gBBTestProtocolInterface;

EFI_STATUS
BBTestMiscRuntimeServicesUnload (
  IN EFI_HANDLE       ImageHandle
  );

EFI_DRIVER_ENTRY_POINT(InitializeBBTestMiscRuntimeServices)

/**
 *  Miscellaneous Runtime Services Test Driver Entry point.
 *  @param ImageHandle the driver image handle.
 *  @param SystemTable the system table.
 *  @return EFI_SUCCESS the driver is loaded successfully.
 */
EFI_STATUS
InitializeBBTestMiscRuntimeServices (
  IN EFI_HANDLE           ImageHandle,
  IN EFI_SYSTEM_TABLE     *SystemTable
  )
{
  EfiInitializeTestLib (ImageHandle, SystemTable);
  InitializeLib (ImageHandle, SystemTable);

  return EfiInitAndInstallBBTestInterface (
           &ImageHandle,
           &gBBTestProtocolField,
           gBBTestEntryField,
           BBTestMiscRuntimeServicesUnload,
           &gBBTestProtocolInterface
           );
}

/**
 *  The driver's Unload function.
 *  @param ImageHandle the test driver image handle.
 *  @return EFI_SUCCESS unload successfully.
 */
EFI_STATUS
BBTestMiscRuntimeServicesUnload (
  IN EFI_HANDLE       ImageHandle
  )
{
  return EfiUninstallAndFreeBBTestInterface (
           ImageHandle,
           gBBTestProtocolInterface
           );
}