/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  EbcDriver.c

Abstract:

--*/

#include "EbcDriver.h"
#include "EfiDriverLib.h"
//#include "EfiPrintLib.h"
#include "EfiCommonLib.h"

EFI_GUID gEfiEbcDriverProtocolGuid = EBC_DRIVER_TEST_PROTOCOL;

//
// Prototypes
//

EFI_STATUS
EbcDriverEntryPoint (
  IN EFI_HANDLE           ImageHandle,
  IN EFI_SYSTEM_TABLE     *SystemTable
  );

EFI_STATUS
EbcDriverUnload (
  IN EFI_HANDLE         ImageHandle
  );

EFI_DRIVER_ENTRY_POINT(EbcDriverEntryPoint)
/**
 *  Ebc Driver Entry point.
 *  @param ImageHandle the driver image handle.
 *  @param SystemTable the system table.
 *  @return  EFI_SUCCESS the driver is loaded successfully.
 *  @return EFI_ALREADY_STARTED the driver has already been loaded before.
 */
EFI_STATUS
EbcDriverEntryPoint (
  IN EFI_HANDLE           ImageHandle,
  IN EFI_SYSTEM_TABLE     *SystemTable
  )
{
  EFI_STATUS                     Status;
  EFI_LOADED_IMAGE_PROTOCOL      *LoadedImage;

  // Initialize driver lib
  EfiInitializeDriverLib (ImageHandle, SystemTable);

  //
  // Fill in the Unload() function
  //
  Status = gBS->OpenProtocol (
                  ImageHandle,
                  &gEfiLoadedImageProtocolGuid,
                  (VOID **)&LoadedImage,
                  ImageHandle,
                  NULL,
                  EFI_OPEN_PROTOCOL_GET_PROTOCOL
                  );

  if (EFI_ERROR (Status)) {
    return Status;
  }

  //
  // The unload function can't invoked in native code
  //
  LoadedImage->Unload = EbcDriverUnload;

  //
  // Open the EBC_DRIVER_TEST_PROTOCOL to perform the supported test.
  //
  Status = gBS->OpenProtocol (
                  ImageHandle,
                  &gEfiEbcDriverProtocolGuid,
                  NULL,
                  ImageHandle,
                  NULL,
                  EFI_OPEN_PROTOCOL_TEST_PROTOCOL
                  );
  if (!EFI_ERROR (Status)) {
    return EFI_ALREADY_STARTED;
  }

  //
  // Install EBC_DRIVER_TEST_PROTOCOL
  //
  Status = gBS->InstallProtocolInterface (
                  &ImageHandle,
                  &gEfiEbcDriverProtocolGuid,
                  EFI_NATIVE_INTERFACE,
                  NULL
                  );

  return Status;
}

/**
 *  Unload function for the driver, uninstall EBC_DRIVER_TEST_PROTOCOL.
 *  @param ImageHandle the driver image handle.
 *  @return  EFI_SUCCESS unload successfully.
 */
EFI_STATUS
EbcDriverUnload (
  IN EFI_HANDLE         ImageHandle
  )
{
  EFI_STATUS                     Status;
  VOID                           *Interface;

  //
  // Open the EBC_DRIVER_TEST_PROTOCOL
  //
  Status = gBS->OpenProtocol (
                  ImageHandle,
                  &gEfiEbcDriverProtocolGuid,
                  &Interface,
                  ImageHandle,
                  NULL,
                  EFI_OPEN_PROTOCOL_GET_PROTOCOL
                  );
  if (!EFI_ERROR (Status)) {
    //
    // Uninstall EBC_DRIVER_TEST_PROTOCOL
    //
    Status = gBS->UninstallProtocolInterface (
                    ImageHandle,
                    &gEfiEbcDriverProtocolGuid,
                    NULL
                    );
  }

  return Status;
}
