/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  LibConfig.h

Abstract:

  This file defines the configuration data structures for the test support
  libraries.

--*/

#ifndef _EFI_LIB_CONFIG_H_
#define _EFI_LIB_CONFIG_H_

//
// Includes
//
#include "Efi.h"
#include "EfiTest.h"
#include EFI_TEST_PROTOCOL_DEFINITION (StandardTestLibrary)
#include EFI_TEST_PROTOCOL_DEFINITION (TestOutputLibrary)

//
// Type definitions
//

//
// Structure for one output file's configuration
//
typedef struct {
  BOOLEAN                             EnableScreenOutput;
  EFI_DEVICE_PATH_PROTOCOL            *DevicePath;
  CHAR16                              *FileName;
  BOOLEAN                             OverwriteFile;
} EFI_LIB_CONFIG_FILE_DATA;

//
// Structure for all logging configuration
//
typedef struct {
  //
  // Test Output Library Protocol
  //
  EFI_TEST_OUTPUT_LIBRARY_PROTOCOL    *OutputProtocol;

  //
  // Log files' configuration
  //
  EFI_LIB_CONFIG_FILE_DATA            SystemLogFile;
  EFI_LIB_CONFIG_FILE_DATA            SystemKeyFile;
  EFI_LIB_CONFIG_FILE_DATA            CaseLogFile;
  EFI_LIB_CONFIG_FILE_DATA            CaseKeyFile;

  //
  // Test platform's configuration
  //
  CHAR16                              *BiosId;
  UINT32                              PlatformNumber;
  UINT32                              ConfigurationNumber;
  CHAR16                              *ScenarioString;

  //
  // Test case's configuration
  //
  UINT64                              TestRevision;
  CHAR16                              *TestName;
  EFI_GUID                            *EntryId;
  CHAR16                              *EntryName;
  CHAR16                              *EntryDescription;
  EFI_GUID                            *SupportProtocols;

  //
  // Test output level
  //
  EFI_VERBOSE_LEVEL                   VerboseLevel;

  //
  // Test status
  //
  BOOLEAN                             IsRecovery;
  UINT32                              PassCount;
  UINT32                              WarningCount;
  UINT32                              FailCount;

  //
  // Additional info to be recorded in the log files
  //
  CHAR16                              *TestCategory;
  CHAR16                              *DevicePath;
} EFI_LIB_CONFIG_DATA;

#endif
