/*++
 Copyright (c) 2008, Intel Corporation
 All rights reserved. 

 This UEFI SCT ("Software") is furnished under license and may only be used or copied
 in accordance with the terms of that license UEFI Contributor Agreement.
 No license, express or implied, by estoppel or otherwise, to any intellectual property
 rights is granted by this document. The Software is subject to change without notice,
 and should not be construed as a commitment by Intel Corporation to market, license,
 sell or support any product or technology. Unless otherwise provided for in the license
 under which this Software is provided, the Software is provided AS IS, with no warranties
 of any kind, express or implied. Except as expressly permitted by the Software license,
 neither Intel Corporation nor its suppliers assumes any responsibility or liability
 for any errors or inaccuracies that may appear herein.
--*/
/*++

Module Name:

  StandardTest.h

Abstract:

  Driver to publish the Standard Test Library Protocol.

--*/

#ifndef _EFI_STANDARD_TEST_H_
#define _EFI_STANDARD_TEST_H_

//
// Includes
//

#include "Efi.h"
#include "EfiTest.h"
#include "LibPrivate.h"
#include EFI_PROTOCOL_DEFINITION (LoadedImage)
#include EFI_TEST_PROTOCOL_DEFINITION (TslInit)
#include EFI_TEST_PROTOCOL_DEFINITION (TestOutputLibrary)
#include EFI_TEST_PROTOCOL_DEFINITION (StandardTestLibrary)

//
// Global definitions
//

#define EFI_MAX_PRINT_BUFFER                768

//
// Private data structures
//

#define STANDARD_TEST_PRIVATE_DATA_SIGNATURE  EFI_SIGNATURE_32('S','T','S','L')

typedef struct {
  BOOLEAN                                   EnableScreenOutput;
  EFI_DEVICE_PATH_PROTOCOL                  *DevicePath;
  CHAR16                                    *FileName;
  BOOLEAN                                   OverwriteFile;
  EFI_FILE_HANDLE                           FileHandle;
} EFI_LIB_CONFIG_FILE_HANDLE;

typedef struct {
  UINT32                                    Signature;
  EFI_STANDARD_TEST_LIBRARY_PROTOCOL        StandardTest;
  EFI_STANDARD_TSL_PRIVATE_INTERFACE        PrivateInterface;
  EFI_TEST_OUTPUT_LIBRARY_PROTOCOL          *OutputProtocol;

  EFI_LIB_CONFIG_FILE_HANDLE                SystemLogFile;
  EFI_LIB_CONFIG_FILE_HANDLE                SystemKeyFile;
  EFI_LIB_CONFIG_FILE_HANDLE                CaseLogFile;
  EFI_LIB_CONFIG_FILE_HANDLE                CaseKeyFile;

  CHAR16                                    *BiosId;
  UINT32                                    PlatformNumber;
  UINT32                                    ConfigurationNumber;
  CHAR16                                    *ScenarioString;

  UINT64                                    TestRevision;
  CHAR16                                    *TestName;
  EFI_GUID                                  EntryId;
  CHAR16                                    *EntryName;
  CHAR16                                    *EntryDescription;
  EFI_GUID                                  *SupportProtocols;
  EFI_VERBOSE_LEVEL                         VerboseLevel;

  BOOLEAN                                   IsRecovery;
  UINT32                                    PassCount;
  UINT32                                    WarningCount;
  UINT32                                    FailCount;

  CHAR16                                    *TestCategory;
  CHAR16                                    *DevicePath;

  BOOLEAN                                   BeginLogging;
  EFI_TIME                                  StartTime;
} STANDARD_TEST_PRIVATE_DATA;

#define STANDARD_TEST_PRIVATE_DATA_FROM_STSL(a) \
  CR(a, STANDARD_TEST_PRIVATE_DATA, StandardTest, STANDARD_TEST_PRIVATE_DATA_SIGNATURE)

#define STANDARD_TEST_PRIVATE_DATA_FROM_PI(a)   \
  CR(a, STANDARD_TEST_PRIVATE_DATA, PrivateInterface, STANDARD_TEST_PRIVATE_DATA_SIGNATURE)


EFI_STATUS
MyUINTToStr (
  IN UINTN        Num,
  OUT CHAR16      *StringNum
);

VOID
StrCpy (
    IN CHAR16   *Dest,
    IN CHAR16   *Src
    );

VOID
StrCat (
    IN CHAR16   *Dest,
    IN CHAR16   *Src
    );


UINTN
StrLen (
    IN CHAR16   *s1
    );
#endif
